@@ -1,6 +1,7 @@
 extern crate proc_macro;
+use fish_tempfile::random_filename;
 use proc_macro::TokenStream;
-use std::{ffi::OsString, fs::OpenOptions, io::Write};
+use std::{ffi::OsString, io::Write, path::PathBuf};
 
 fn unescape_multiline_rust_string(s: String) -> String {
     if !s.contains('\n') {
@@ -42,12 +43,9 @@ fn unescape_multiline_rust_string(s: String) -> String {
     unescaped
 }
 
-fn append_po_entry_to_file(message: &TokenStream, file_name: &OsString) {
-    let mut file = OpenOptions::new()
-        .create(true)
-        .append(true)
-        .open(file_name)
-        .unwrap_or_else(|e| panic!("Could not open file {file_name:?}: {e}"));
+// Each entry is written to a fresh file to avoid race conditions arising when there are multiple
+// unsynchronized writers to the same file.
+fn write_po_entry_to_file(message: &TokenStream, dir: &OsString) {
     let message_string = unescape_multiline_rust_string(message.to_string());
     if message_string.contains('\n') {
         panic!(
@@ -61,25 +59,32 @@ fn append_po_entry_to_file(message: &TokenStream, file_name: &OsString) {
         ""
     };
     let po_entry = format!("{format_string_annotation}msgid {message_string}\nmsgstr \"\"\n\n");
+
+    let dir = PathBuf::from(dir);
+    let (path, result) =
+        fish_tempfile::create_file_with_retry(|| dir.join(random_filename(OsString::new())));
+    let mut file = result.unwrap_or_else(|e| {
+        panic!("Failed to create temporary file {path:?}:\n{e}");
+    });
     file.write_all(po_entry.as_bytes()).unwrap();
 }
 
 /// The `message` is passed through unmodified.
-/// If `FISH_GETTEXT_EXTRACTION_FILE` is defined in the environment,
-/// this file is used to write the message,
+/// If `FISH_GETTEXT_EXTRACTION_DIR` is defined in the environment,
+/// the message ID is written into a new file in this directory,
 /// so that it can then be used for generating gettext PO files.
 /// The `message` must be a string literal.
 ///
 /// # Panics
 ///
-/// This macro panics if the `FISH_GETTEXT_EXTRACTION_FILE` variable is set and `message` has an
+/// This macro panics if the `FISH_GETTEXT_EXTRACTION_DIR` variable is set and `message` has an
 /// unexpected format.
 /// Note that for example `concat!(...)` cannot be passed to this macro, because expansion works
 /// outside in, meaning this macro would still see the `concat!` macro invocation, instead of a
 /// string literal.
 #[proc_macro]
 pub fn gettext_extract(message: TokenStream) -> TokenStream {
-    if let Some(file_path) = std::env::var_os("FISH_GETTEXT_EXTRACTION_FILE") {
+    if let Some(dir_path) = std::env::var_os("FISH_GETTEXT_EXTRACTION_DIR") {
         let pm2_message = proc_macro2::TokenStream::from(message.clone());
         let mut token_trees = pm2_message.into_iter();
         let first_token = token_trees
@@ -103,7 +108,7 @@ pub fn gettext_extract(message: TokenStream) -> TokenStream {
             )
         }
         if let proc_macro2::TokenTree::Literal(_) = first_group_token {
-            append_po_entry_to_file(&message, &file_path);
+            write_po_entry_to_file(&message, &dir_path);
         } else {
             panic!("Expected literal in gettext_extract, but got: {first_group_token:?}");
         }
