@@ -1,7 +1,6 @@
 use crate::arg::ToArg;
 use crate::locale::{C_LOCALE, EN_US_LOCALE, Locale};
 use crate::{Error, FormatString, sprintf_locale};
-use libc::c_char;
 use std::f64::consts::{E, PI, TAU};
 use std::ffi::CStr;
 use std::fmt;
@@ -890,10 +889,16 @@ fn libc_sprintf_one_float_with_precision<'a>(
     fmt: &'a CStr,
 ) -> impl FnMut(usize, f64) -> &'a str {
     |preci, float_val| unsafe {
-        let storage_ptr = storage.as_mut_ptr() as *mut c_char;
-        let len = libc::snprintf(storage_ptr, storage.len(), fmt.as_ptr(), preci, float_val);
+        let storage_ptr = storage.as_mut_ptr();
+        let len = libc::snprintf(
+            storage_ptr.cast(),
+            storage.len(),
+            fmt.as_ptr(),
+            preci,
+            float_val,
+        );
         assert!(len >= 0);
-        let sl = std::slice::from_raw_parts(storage_ptr as *const u8, len as usize);
+        let sl = std::slice::from_raw_parts(storage_ptr, len as usize);
         std::str::from_utf8(sl).unwrap()
     }
 }
