@@ -157,9 +157,9 @@ impl BuiltinBind {
             let mut colors = Vec::new();
             highlight_shell(&out, &mut colors, &parser.context(), false, None);
             let colored = colorize(&out, &colors, parser.vars());
-            streams.out.append(bytes2wcstring(&colored));
+            streams.out.append(&bytes2wcstring(&colored));
         } else {
-            streams.out.append(out);
+            streams.out.append(&out);
         }
 
         true
@@ -251,7 +251,7 @@ impl BuiltinBind {
         match parse_keys(seq) {
             Ok(keys) => Some(keys),
             Err(err) => {
-                streams.err.append(sprintf!("bind: %s\n", err));
+                streams.err.append(&sprintf!("bind: %s\n", err));
                 None
             }
         }
@@ -306,7 +306,7 @@ impl BuiltinBind {
         } else {
             // Inserting both on the other hand makes no sense.
             if self.opts.have_preset && self.opts.have_user {
-                streams.err.append(wgettext_fmt!(
+                streams.err.append(&wgettext_fmt!(
                     BUILTIN_ERR_COMBO2_EXCLUSIVE,
                     cmd,
                     "--preset",
@@ -349,13 +349,13 @@ impl BuiltinBind {
                 );
                 if !self.opts.silent {
                     if seq.len() == 1 {
-                        streams.err.append(wgettext_fmt!(
+                        streams.err.append(&wgettext_fmt!(
                             "%s: No binding found for key '%s'\n",
                             cmd,
                             seq[0]
                         ));
                     } else {
-                        streams.err.append(wgettext_fmt!(
+                        streams.err.append(&wgettext_fmt!(
                             "%s: No binding found for key sequence '%s'\n",
                             cmd,
                             eseq
@@ -423,6 +423,19 @@ fn parse_cmd_opts(
         wopt(L!("user"), NoArgument, 'u'),
     ];
 
+    let mut check_mode_name = |mode_name: &wstr| -> Result<(), ErrorCode> {
+        if !valid_var_name(mode_name) {
+            streams.err.append(&wgettext_fmt!(
+                BUILTIN_ERR_BIND_MODE,
+                cmd,
+                mode_name,
+                help_section!("language#shell-variable-and-function-names")
+            ));
+            return Err(STATUS_INVALID_ARGS);
+        }
+        Ok(())
+    };
+
     let mut w = WGetopter::new(short_options, long_options, argv);
     while let Some(c) = w.next_opt() {
         match c {
@@ -431,7 +444,7 @@ fn parse_cmd_opts(
             'f' => opts.mode = BIND_FUNCTION_NAMES,
             'h' => opts.print_help = true,
             'k' => {
-                streams.err.append(wgettext_fmt!(
+                streams.err.append(&wgettext_fmt!(
                     "%s: the -k/--key syntax is no longer supported. See `bind --help` and `bind --key-names`\n",
                     cmd,
                 ));
@@ -443,25 +456,14 @@ fn parse_cmd_opts(
                 return Ok(SUCCESS);
             }
             'M' => {
-                if !valid_var_name(w.woptarg.unwrap()) {
-                    streams.err.append(wgettext_fmt!(
-                        BUILTIN_ERR_BIND_MODE,
-                        cmd,
-                        w.woptarg.unwrap()
-                    ));
-                    return Err(STATUS_INVALID_ARGS);
-                }
-                opts.bind_mode = w.woptarg.unwrap().to_owned();
+                let applicable_mode = w.woptarg.unwrap();
+                check_mode_name(applicable_mode)?;
+                opts.bind_mode = applicable_mode.to_owned();
                 opts.bind_mode_given = true;
             }
             'm' => {
                 let new_mode = w.woptarg.unwrap();
-                if !valid_var_name(new_mode) {
-                    streams
-                        .err
-                        .append(wgettext_fmt!(BUILTIN_ERR_BIND_MODE, cmd, new_mode));
-                    return Err(STATUS_INVALID_ARGS);
-                }
+                check_mode_name(new_mode)?;
                 opts.sets_bind_mode = Some(new_mode.to_owned());
             }
             'p' => {
@@ -491,7 +493,7 @@ fn parse_cmd_opts(
         }
     }
     *optind = w.wopt_index;
-    return Ok(SUCCESS);
+    Ok(SUCCESS)
 }
 
 impl BuiltinBind {
@@ -555,7 +557,7 @@ impl BuiltinBind {
             _ => {
                 streams
                     .err
-                    .append(wgettext_fmt!("%s: Invalid state\n", cmd));
+                    .append(&wgettext_fmt!("%s: Invalid state\n", cmd));
                 return Err(STATUS_CMD_ERROR);
             }
         }
