@@ -14,8 +14,8 @@ pub fn r#command(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) -
     let print_hints = false;
     let mut opts: command_cmd_opts_t = Default::default();
 
-    const shortopts: &wstr = L!("hasqv");
-    const longopts: &[WOption] = &[
+    let shortopts: &wstr = L!("hasqv");
+    let longopts: &[WOption] = &[
         wopt(L!("help"), ArgType::NoArgument, 'h'),
         wopt(L!("all"), ArgType::NoArgument, 'a'),
         wopt(L!("query"), ArgType::NoArgument, 'q'),
