@@ -146,6 +146,7 @@ fn builtin_complete_remove_cmd(
     short_opt: &wstr,
     gnu_opt: &[&wstr],
     old_opt: &[&wstr],
+    wrap_targets: &[WString],
 ) {
     let mut removed = false;
     for s in short_opt.chars() {
@@ -178,6 +179,11 @@ fn builtin_complete_remove_cmd(
         removed = true;
     }
 
+    for wrap_target in wrap_targets {
+        complete_remove_wrapper(cmd.to_owned(), wrap_target);
+        removed = true;
+    }
+
     if !removed {
         // This means that all loops were empty.
         complete_remove_all(cmd.to_owned(), cmd_is_path, /*explicit=*/ true);
@@ -190,13 +196,28 @@ fn builtin_complete_remove(
     short_opt: &wstr,
     gnu_opt: &[&wstr],
     old_opt: &[&wstr],
+    wrap_targets: &[WString],
 ) {
     for cmd in cmds {
-        builtin_complete_remove_cmd(cmd, false /* not path */, short_opt, gnu_opt, old_opt);
+        builtin_complete_remove_cmd(
+            cmd,
+            false, /* not path */
+            short_opt,
+            gnu_opt,
+            old_opt,
+            wrap_targets,
+        );
     }
 
     for path in paths {
-        builtin_complete_remove_cmd(path, true /* is path */, short_opt, gnu_opt, old_opt);
+        builtin_complete_remove_cmd(
+            path,
+            true, /* is path */
+            short_opt,
+            gnu_opt,
+            old_opt,
+            wrap_targets,
+        );
     }
 }
 
@@ -209,9 +230,9 @@ fn builtin_complete_print(cmd: &wstr, streams: &mut IoStreams, parser: &Parser)
         highlight_shell(&repr, &mut colors, &parser.context(), false, None);
         streams
             .out
-            .append(bytes2wcstring(&colorize(&repr, &colors, parser.vars())));
+            .append(&bytes2wcstring(&colorize(&repr, &colors, parser.vars())));
     } else {
-        streams.out.append(repr);
+        streams.out.append(&repr);
     }
 }
 
@@ -297,7 +318,7 @@ pub fn complete(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) ->
                         cmd_to_complete.push(tmp);
                     }
                 } else {
-                    streams.err.append(wgettext_fmt!(
+                    streams.err.append(&wgettext_fmt!(
                         "%s: Invalid token '%s'\n",
                         cmd,
                         w.woptarg.unwrap()
@@ -320,7 +341,7 @@ pub fn complete(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) ->
                 if arg.is_empty() {
                     streams
                         .err
-                        .append(wgettext_fmt!("%s: -s requires a non-empty string\n", cmd,));
+                        .append(&wgettext_fmt!("%s: -s requires a non-empty string\n", cmd,));
                     return Err(STATUS_INVALID_ARGS);
                 }
             }
@@ -330,7 +351,7 @@ pub fn complete(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) ->
                 if arg.is_empty() {
                     streams
                         .err
-                        .append(wgettext_fmt!("%s: -l requires a non-empty string\n", cmd,));
+                        .append(&wgettext_fmt!("%s: -l requires a non-empty string\n", cmd,));
                     return Err(STATUS_INVALID_ARGS);
                 }
             }
@@ -340,7 +361,7 @@ pub fn complete(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) ->
                 if arg.is_empty() {
                     streams
                         .err
-                        .append(wgettext_fmt!("%s: -o requires a non-empty string\n", cmd,));
+                        .append(&wgettext_fmt!("%s: -o requires a non-empty string\n", cmd,));
                     return Err(STATUS_INVALID_ARGS);
                 }
             }
@@ -386,7 +407,7 @@ pub fn complete(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) ->
 
     if result_mode.no_files && result_mode.force_files {
         if !have_x {
-            streams.err.append(wgettext_fmt!(
+            streams.err.append(&wgettext_fmt!(
                 BUILTIN_ERR_COMBO2,
                 "complete",
                 "'--no-files' and '--force-files'"
@@ -394,7 +415,7 @@ pub fn complete(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) ->
         } else {
             // The reason for us not wanting files is `-x`,
             // which is short for `-rf`.
-            streams.err.append(wgettext_fmt!(
+            streams.err.append(&wgettext_fmt!(
                 BUILTIN_ERR_COMBO2,
                 "complete",
                 "'--exclusive' and '--force-files'"
@@ -414,7 +435,7 @@ pub fn complete(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) ->
         } else {
             streams
                 .err
-                .append(wgettext_fmt!(BUILTIN_ERR_TOO_MANY_ARGUMENTS, cmd));
+                .append(&wgettext_fmt!(BUILTIN_ERR_TOO_MANY_ARGUMENTS, cmd));
             builtin_print_error_trailer(parser, streams.err, cmd);
             return Err(STATUS_INVALID_ARGS);
         }
@@ -425,13 +446,13 @@ pub fn complete(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) ->
         if parse_util_detect_errors(condition_string, Some(&mut errors), false).is_err() {
             for error in errors {
                 let prefix = cmd.to_owned() + L!(": -n '") + &condition_string[..] + L!("': ");
-                streams.err.append(error.describe_with_prefix(
+                streams.err.append(&error.describe_with_prefix(
                     condition_string,
                     &prefix,
                     parser.is_interactive(),
                     false,
                 ));
-                streams.err.push('\n');
+                streams.err.append_char('\n');
             }
             return Err(STATUS_CMD_ERROR);
         }
@@ -443,13 +464,13 @@ pub fn complete(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) ->
         prefix.push_str(": ");
 
         if let Err(err_text) = parse_util_detect_errors_in_argument_list(&comp, &prefix) {
-            streams.err.append(wgettext_fmt!(
+            streams.err.append(&wgettext_fmt!(
                 "%s: %s: contains a syntax error\n",
                 cmd,
                 comp
             ));
-            streams.err.append(err_text);
-            streams.err.push('\n');
+            streams.err.append(&err_text);
+            streams.err.append_char('\n');
             return Err(STATUS_CMD_ERROR);
         }
     }
@@ -542,7 +563,7 @@ pub fn complete(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) ->
                     faux_cmdline_with_completion.push_utfstr(&next.description);
                 }
                 faux_cmdline_with_completion.push('\n');
-                streams.out.append(faux_cmdline_with_completion);
+                streams.out.append(&faux_cmdline_with_completion);
             }
 
             parser.libdata_mut().builtin_complete_current_commandline = false;
@@ -579,7 +600,14 @@ pub fn complete(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) ->
         }
 
         if remove {
-            builtin_complete_remove(&cmd_to_complete, &path, &short_opt, &gnu_opt, &old_opt);
+            builtin_complete_remove(
+                &cmd_to_complete,
+                &path,
+                &short_opt,
+                &gnu_opt,
+                &old_opt,
+                &wrap_targets,
+            );
         } else {
             builtin_complete_add(
                 &cmd_to_complete,
@@ -593,14 +621,9 @@ pub fn complete(parser: &Parser, streams: &mut IoStreams, argv: &mut [&wstr]) ->
                 &desc,
                 flags,
             );
-        }
-
-        // Handle wrap targets (probably empty). We only wrap commands, not paths.
-        for wrap_target in wrap_targets {
-            for i in &cmd_to_complete {
-                if remove {
-                    complete_remove_wrapper(i.clone(), &wrap_target);
-                } else {
+            // Handle wrap targets (probably empty). We only wrap commands, not paths.
+            for wrap_target in wrap_targets {
+                for i in &cmd_to_complete {
                     complete_add_wrapper(i.clone(), wrap_target.clone());
                 }
             }
