@@ -55,10 +55,10 @@ mod prelude {
 
     #[allow(unused_imports)]
     pub(crate) use crate::{
-        flog::{FLOG, FLOGF},
+        flog::{flog, flogf},
         io::{IoStreams, SeparationType},
         parser::Parser,
-        wchar::prelude::*,
+        prelude::*,
         wgetopt::{
             ArgType::{self, *},
             NON_OPTION_CHAR, WGetopter, WOption, wopt,
