@@ -2,12 +2,14 @@ use crate::common::init_special_chars_once;
 use crate::complete::complete_invalidate_path;
 use crate::env::{DEFAULT_READ_BYTE_LIMIT, READ_BYTE_LIMIT};
 use crate::env::{EnvMode, EnvStack, Environment, setenv_lock, unsetenv_lock};
-use crate::flog::FLOG;
+use crate::flog::flog;
 use crate::input_common::{update_wait_on_escape_ms, update_wait_on_sequence_key_ms};
 use crate::locale::{invalidate_numeric_locale, set_libc_locales};
+use crate::prelude::*;
 use crate::reader::{
     reader_change_cursor_end_mode, reader_change_cursor_selection_mode, reader_change_history,
-    reader_schedule_prompt_repaint, reader_set_autosuggestion_enabled, reader_set_transient_prompt,
+    reader_current_data, reader_schedule_prompt_repaint, reader_set_autosuggestion_enabled,
+    reader_set_transient_prompt,
 };
 use crate::screen::{
     IS_DUMB, LAYOUT_CACHE_SHARED, ONLY_GRAYSCALE, screen_set_midnight_commander_hack,
@@ -15,7 +17,7 @@ use crate::screen::{
 use crate::terminal::ColorSupport;
 use crate::terminal::use_terminfo;
 use crate::tty_handoff::xtversion;
-use crate::wchar::prelude::*;
+use crate::wcstringutil::string_prefixes_string;
 use crate::wutil::fish_wcstoi;
 use crate::{function, terminal};
 use std::collections::HashMap;
@@ -46,8 +48,14 @@ static VAR_DISPATCH_TABLE: once_cell::sync::Lazy<VarDispatchTable> =
     once_cell::sync::Lazy::new(|| {
         let mut table = VarDispatchTable::default();
 
+        macro_rules! vars {
+            ( $f:ident ) => {
+                |vars: &EnvStack, _suppress_repaint: bool| $f(vars)
+            };
+        }
+
         for name in LOCALE_VARIABLES {
-            table.add_anon(name, handle_locale_change);
+            table.add_anon(name, vars!(handle_locale_change));
         }
 
         for name in CURSES_VARIABLES {
@@ -58,43 +66,49 @@ static VAR_DISPATCH_TABLE: once_cell::sync::Lazy<VarDispatchTable> =
         table.add_anon(L!("COLORTERM"), handle_fish_term_change);
         table.add_anon(L!("fish_term256"), handle_fish_term_change);
         table.add_anon(L!("fish_term24bit"), handle_fish_term_change);
-        table.add_anon(L!("fish_escape_delay_ms"), update_wait_on_escape_ms);
+        table.add_anon(L!("fish_escape_delay_ms"), vars!(update_wait_on_escape_ms));
         table.add_anon(
             L!("fish_sequence_key_delay_ms"),
-            update_wait_on_sequence_key_ms,
+            vars!(update_wait_on_sequence_key_ms),
         );
-        table.add_anon(L!("fish_emoji_width"), guess_emoji_width);
-        table.add_anon(L!("fish_ambiguous_width"), handle_change_ambiguous_width);
-        table.add_anon(L!("LINES"), handle_term_size_change);
-        table.add_anon(L!("COLUMNS"), handle_term_size_change);
-        table.add_anon(L!("fish_complete_path"), handle_complete_path_change);
-        table.add_anon(L!("fish_function_path"), handle_function_path_change);
-        table.add_anon(L!("fish_read_limit"), handle_read_limit_change);
-        table.add_anon(L!("fish_history"), handle_fish_history_change);
+        table.add_anon(L!("fish_emoji_width"), vars!(guess_emoji_width));
+        table.add_anon(
+            L!("fish_ambiguous_width"),
+            vars!(handle_change_ambiguous_width),
+        );
+        table.add_anon(L!("LINES"), vars!(handle_term_size_change));
+        table.add_anon(L!("COLUMNS"), vars!(handle_term_size_change));
+        table.add_anon(L!("fish_complete_path"), vars!(handle_complete_path_change));
+        table.add_anon(L!("fish_function_path"), vars!(handle_function_path_change));
+        table.add_anon(L!("fish_read_limit"), vars!(handle_read_limit_change));
+        table.add_anon(L!("fish_history"), vars!(handle_fish_history_change));
         table.add_anon(
             L!("fish_autosuggestion_enabled"),
-            handle_autosuggestion_change,
+            vars!(handle_autosuggestion_change),
+        );
+        table.add_anon(
+            L!("fish_transient_prompt"),
+            vars!(handle_transient_prompt_change),
         );
-        table.add_anon(L!("fish_transient_prompt"), handle_transient_prompt_change);
         table.add_anon(
             L!("fish_use_posix_spawn"),
-            handle_fish_use_posix_spawn_change,
+            vars!(handle_fish_use_posix_spawn_change),
         );
-        table.add_anon(L!("fish_trace"), handle_fish_trace);
+        table.add_anon(L!("fish_trace"), vars!(handle_fish_trace));
         table.add_anon(
             L!("fish_cursor_selection_mode"),
-            handle_fish_cursor_selection_mode_change,
+            vars!(handle_fish_cursor_selection_mode_change),
         );
         table.add_anon(
             L!("fish_cursor_end_mode"),
-            handle_fish_cursor_end_mode_change,
+            vars!(handle_fish_cursor_end_mode_change),
         );
 
         table
     });
 
 type NamedEnvCallback = fn(name: &wstr, env: &EnvStack);
-type AnonEnvCallback = fn(env: &EnvStack);
+type AnonEnvCallback = fn(env: &EnvStack, suppress_repaint: bool);
 
 enum EnvCallback {
     Named(NamedEnvCallback),
@@ -119,10 +133,10 @@ impl VarDispatchTable {
         assert!(prev.is_none(), "Already observing {}", name);
     }
 
-    pub fn dispatch(&self, key: &wstr, vars: &EnvStack) {
+    pub fn dispatch(&self, key: &wstr, vars: &EnvStack, suppress_repaint: bool) {
         match self.table.get(key) {
             Some(EnvCallback::Named(named)) => (named)(key, vars),
-            Some(EnvCallback::Anon(anon)) => (anon)(vars),
+            Some(EnvCallback::Anon(anon)) => (anon)(vars, suppress_repaint),
             None => (),
         }
     }
@@ -130,7 +144,7 @@ impl VarDispatchTable {
 
 fn handle_timezone(var_name: &wstr, vars: &EnvStack) {
     let var = vars.get_unless_empty(var_name).map(|v| v.as_string());
-    FLOG!(
+    flog!(
         env_dispatch,
         "handle_timezone() current timezone var:",
         var_name,
@@ -154,15 +168,15 @@ fn handle_timezone(var_name: &wstr, vars: &EnvStack) {
     }
 }
 
-/// Update the value of [`FISH_EMOJI_WIDTH`](crate::fallback::FISH_EMOJI_WIDTH).
+/// Update the value of [`FISH_EMOJI_WIDTH`](fish_fallback::FISH_EMOJI_WIDTH).
 pub fn guess_emoji_width(vars: &EnvStack) {
-    use crate::fallback::FISH_EMOJI_WIDTH;
+    use fish_fallback::FISH_EMOJI_WIDTH;
 
     if let Some(width_str) = vars.get(L!("fish_emoji_width")) {
         // The only valid values are 1 or 2; we default to 1 if it was an invalid int.
         let new_width = fish_wcstoi(&width_str.as_string()).unwrap_or(1).clamp(1, 2) as isize;
         FISH_EMOJI_WIDTH.store(new_width, Ordering::Relaxed);
-        FLOG!(
+        flog!(
             term_support,
             "Overriding default fish_emoji_width w/",
             new_width
@@ -172,13 +186,13 @@ pub fn guess_emoji_width(vars: &EnvStack) {
 
     let term_program = vars
         .get(L!("TERM_PROGRAM"))
-        .map(|v| v.as_string())
-        .unwrap_or_else(WString::new);
+        .map_or_else(WString::new, |v| v.as_string());
 
+    // TODO(term-workaround)
     if xtversion().unwrap_or(L!("")).starts_with(L!("iTerm2 ")) {
         // iTerm2 now defaults to Unicode 9 sizes for anything after macOS 10.12
         FISH_EMOJI_WIDTH.store(2, Ordering::Relaxed);
-        FLOG!(term_support, "default emoji width 2 for iTerm2");
+        flog!(term_support, "default emoji width 2 for iTerm2");
     } else if term_program == "Apple_Terminal" && {
         let version = vars
             .get(L!("TERM_PROGRAM_VERSION"))
@@ -192,32 +206,53 @@ pub fn guess_emoji_width(vars: &EnvStack) {
     } {
         // Apple Terminal on High Sierra
         FISH_EMOJI_WIDTH.store(2, Ordering::Relaxed);
-        FLOG!(term_support, "default emoji width: 2 for", term_program);
+        flog!(term_support, "default emoji width: 2 for", term_program);
     } else {
         // Default to whatever the system's wcwidth gives for U+1F603, but only if it's at least
         // 1 and at most 2.
         #[cfg(not(cygwin))]
-        let width = crate::fallback::wcwidth('😃').clamp(1, 2);
+        let width = fish_fallback::wcwidth('😃').clamp(1, 2);
         #[cfg(cygwin)]
         let width = 2_isize;
         FISH_EMOJI_WIDTH.store(width, Ordering::Relaxed);
-        FLOG!(term_support, "default emoji width:", width);
+        flog!(term_support, "default emoji width:", width);
     }
 }
 
+pub struct VarChangeMilieu {
+    pub is_repainting: bool,
+    pub global_or_universal: bool,
+}
+
 /// React to modifying the given variable.
-pub fn env_dispatch_var_change(key: &wstr, vars: &EnvStack) {
+pub fn env_dispatch_var_change(milieu: VarChangeMilieu, key: &wstr, vars: &EnvStack) {
     use once_cell::sync::Lazy;
 
+    let suppress_repaint = milieu.is_repainting || !milieu.global_or_universal;
+
     // We want to ignore variable changes until the dispatch table is explicitly initialized.
     if let Some(dispatch_table) = Lazy::get(&VAR_DISPATCH_TABLE) {
-        dispatch_table.dispatch(key, vars);
+        dispatch_table.dispatch(key, vars, suppress_repaint);
+    }
+
+    // TODO(MSRV>=1.88): if-let
+    if !suppress_repaint {
+        if let Some(data) = reader_current_data() {
+            if string_prefixes_string(L!("fish_color_"), key) || {
+                // TODO Don't re-exec prompt when only pager color changed.
+                string_prefixes_string(L!("fish_pager_color_"), key)
+            } {
+                data.schedule_prompt_repaint();
+            }
+        }
     }
 }
 
-fn handle_fish_term_change(vars: &EnvStack) {
+fn handle_fish_term_change(vars: &EnvStack, suppress_repaint: bool) {
     update_fish_color_support(vars);
-    reader_schedule_prompt_repaint();
+    if !suppress_repaint {
+        reader_schedule_prompt_repaint();
+    }
 }
 
 fn handle_change_ambiguous_width(vars: &EnvStack) {
@@ -229,7 +264,7 @@ fn handle_change_ambiguous_width(vars: &EnvStack) {
         .unwrap_or(1)
         // Clamp in case of negative values.
         .max(0) as isize;
-    crate::fallback::FISH_AMBIGUOUS_WIDTH.store(new_width, Ordering::Relaxed);
+    fish_fallback::FISH_AMBIGUOUS_WIDTH.store(new_width, Ordering::Relaxed);
 }
 
 fn handle_term_size_change(vars: &EnvStack) {
@@ -248,8 +283,7 @@ fn handle_fish_cursor_selection_mode_change(vars: &EnvStack) {
         .get(L!("fish_cursor_selection_mode"))
         .as_ref()
         .map(|v| v.as_string())
-        .map(|v| v == "inclusive")
-        .unwrap_or(false);
+        .is_some_and(|v| v == "inclusive");
     let mode = if inclusive {
         CursorSelectionMode::Inclusive
     } else {
@@ -266,8 +300,7 @@ fn handle_fish_cursor_end_mode_change(vars: &EnvStack) {
         .get(L!("fish_cursor_end_mode"))
         .as_ref()
         .map(|v| v.as_string())
-        .map(|v| v == "inclusive")
-        .unwrap_or(false);
+        .is_some_and(|v| v == "inclusive");
     let mode = if inclusive {
         CursorEndMode::Inclusive
     } else {
@@ -301,16 +334,18 @@ fn handle_locale_change(vars: &EnvStack) {
     init_locale(vars);
 }
 
-fn handle_term_change(vars: &EnvStack) {
+fn handle_term_change(vars: &EnvStack, suppress_repaint: bool) {
     guess_emoji_width(vars);
     init_terminal(vars);
     read_terminfo_database(vars);
-    reader_schedule_prompt_repaint();
+    if !suppress_repaint {
+        reader_schedule_prompt_repaint();
+    }
 }
 
 fn handle_fish_use_posix_spawn_change(vars: &EnvStack) {
     // Note that if the variable is missing or empty we default to true (if allowed).
-    if !allow_use_posix_spawn() {
+    if !cfg!(have_posix_spawn) {
         USE_POSIX_SPAWN.store(false, Ordering::Relaxed);
     } else if let Some(var) = vars.get(L!("fish_use_posix_spawn")) {
         let use_posix_spawn =
@@ -336,7 +371,7 @@ fn handle_read_limit_change(vars: &EnvStack) {
                 Some(v) => Some(v),
                 None => {
                     // We intentionally warn here even in non-interactive mode.
-                    FLOG!(warning, "Ignoring invalid $fish_read_limit");
+                    flog!(warning, "Ignoring invalid $fish_read_limit");
                     None
                 }
             }
@@ -349,8 +384,11 @@ fn handle_read_limit_change(vars: &EnvStack) {
 }
 
 fn handle_fish_trace(vars: &EnvStack) {
-    let enabled = vars.get_unless_empty(L!("fish_trace")).is_some();
-    crate::trace::trace_set_enabled(enabled);
+    crate::trace::trace_set_enabled(
+        vars.get_unless_empty(L!("fish_trace"))
+            .map(|var| var.as_list().to_vec())
+            .unwrap_or_default(),
+    );
 }
 
 pub fn env_dispatch_init(vars: &EnvStack) {
@@ -386,7 +424,7 @@ fn update_fish_color_support(vars: &EnvStack) {
 
     let supports_256color = if let Some(fish_term256) = vars.get(L!("fish_term256")) {
         let ok = crate::wcstringutil::bool_from_string(&fish_term256.as_string());
-        FLOG!(
+        flog!(
             term_support,
             "256-color support determined by $fish_term256:",
             ok
@@ -397,10 +435,11 @@ fn update_fish_color_support(vars: &EnvStack) {
     };
 
     let supports_24bit;
+    #[allow(unused_parens)]
     if let Some(fish_term24bit) = vars.get(L!("fish_term24bit")).map(|v| v.as_string()) {
         // $fish_term24bit
         supports_24bit = crate::wcstringutil::bool_from_string(&fish_term24bit);
-        FLOG!(
+        flog!(
             term_support,
             "$fish_term24bit preference: 24-bit color",
             if supports_24bit {
@@ -409,15 +448,18 @@ fn update_fish_color_support(vars: &EnvStack) {
                 "disabled"
             }
         );
-    } else if vars.get(L!("STY")).is_some() {
+    } else if (
+        // TODO(term-workaround)
+        vars.get(L!("STY")).is_some()
+    ) {
         // Screen requires "truecolor on" to enable true-color sequences, so we ignore them
         // unless force-enabled.
         supports_24bit = false;
-        FLOG!(term_support, "True-color support: disabled for screen");
+        flog!(term_support, "True-color support: disabled for screen");
     } else if let Some(ct) = vars.get(L!("COLORTERM")).map(|v| v.as_string()) {
         // If someone sets $COLORTERM, that's the sort of color they want.
         supports_24bit = ct == "truecolor" || ct == "24bit";
-        FLOG!(
+        flog!(
             term_support,
             "True-color support",
             if supports_24bit {
@@ -429,11 +471,12 @@ fn update_fish_color_support(vars: &EnvStack) {
             ct
         );
     } else {
-        supports_24bit = !is_xterm_16color
-            && vars
-                .get_unless_empty(L!("TERM_PROGRAM"))
-                .is_none_or(|term| term.as_list()[0] != "Apple_Terminal");
-        FLOG!(
+        supports_24bit = !is_xterm_16color && {
+            // TODO(term-workaround)
+            vars.get_unless_empty(L!("TERM_PROGRAM"))
+                .is_none_or(|term| term.as_list()[0] != "Apple_Terminal")
+        };
+        flog!(
             term_support,
             "True-color support",
             if supports_24bit {
@@ -464,6 +507,7 @@ fn init_terminal(vars: &EnvStack) {
     IS_DUMB.store(term == "dumb");
     ONLY_GRAYSCALE.store(term == "ansi-m" || term == "linux-m" || term == "xterm-mono");
 
+    // TODO(term-workaround)
     if vars.get(MIDNIGHT_COMMANDER_SID).is_some() {
         screen_set_midnight_commander_hack();
     }
@@ -483,10 +527,10 @@ pub fn read_terminfo_database(vars: &EnvStack) {
             .getf_unless_empty(var_name, EnvMode::EXPORT)
             .map(|v| v.as_string())
         {
-            FLOG!(term_support, "curses var", var_name, "=", value);
+            flog!(term_support, "curses var", var_name, "=", value);
             setenv_lock(var_name, &value, true);
         } else {
-            FLOG!(term_support, "curses var", var_name, "is missing or empty");
+            flog!(term_support, "curses var", var_name, "is missing or empty");
             unsetenv_lock(var_name);
         }
     }
@@ -506,10 +550,10 @@ fn init_locale(vars: &EnvStack) {
             .getf_unless_empty(var_name, EnvMode::EXPORT)
             .map(|v| v.as_string());
         if let Some(value) = var {
-            FLOG!(env_locale, "locale var", var_name, "=", value);
+            flog!(env_locale, "locale var", var_name, "=", value);
             setenv_lock(var_name, &value, true);
         } else {
-            FLOG!(env_locale, "locale var", var_name, "is missing or empty");
+            flog!(env_locale, "locale var", var_name, "is missing or empty");
             unsetenv_lock(var_name);
         }
     }
@@ -518,33 +562,16 @@ fn init_locale(vars: &EnvStack) {
     if !unsafe {
         set_libc_locales(/*log_ok=*/ true)
     } {
-        FLOG!(env_locale, "user has an invalid locale configured");
+        flog!(env_locale, "user has an invalid locale configured");
     }
 
     // Invalidate the cached numeric locale.
     invalidate_numeric_locale();
 
     #[cfg(feature = "localize-messages")]
-    crate::wutil::gettext::update_locale_from_env(vars);
+    crate::localization::update_from_env(vars);
 }
 
 pub fn use_posix_spawn() -> bool {
     USE_POSIX_SPAWN.load(Ordering::Relaxed)
 }
-
-/// Whether or not we are running on an OS where we allow ourselves to use `posix_spawn()`.
-const fn allow_use_posix_spawn() -> bool {
-    // OpenBSD's posix_spawn returns status 127 instead of erroring with ENOEXEC when faced with a
-    // shebang-less script. Disable posix_spawn on OpenBSD.
-    if cfg!(target_os = "openbsd") {
-        false
-    } else if cfg!(not(target_os = "linux")) {
-        true
-    } else {
-        // The C++ code used __GLIBC_PREREQ(2, 24) && !defined(__UCLIBC__) to determine if we'll use
-        // posix_spawn() by default on Linux. Surprise! We don't have to worry about porting that
-        // logic here because the libc crate only supports 2.26+ atm.
-        // See https://github.com/rust-lang/libc/issues/1412
-        true
-    }
-}
