@@ -1,3 +1,4 @@
+use cfg_if::cfg_if;
 #[cfg(not(target_has_atomic = "64"))]
 use portable_atomic::AtomicU64;
 use std::collections::HashMap;
@@ -10,17 +11,19 @@ use std::time::Duration;
 
 use crate::common::exit_without_destructors;
 use crate::fd_readable_set::{FdReadableSet, Timeout};
-use crate::fds::AutoCloseFd;
-use crate::flog::FLOG;
+use crate::flog::flog;
 use crate::threads::assert_is_background_thread;
 use crate::wutil::perror;
 use errno::errno;
-use libc::{EAGAIN, EINTR, EWOULDBLOCK, c_void};
+use libc::{EAGAIN, EINTR, EWOULDBLOCK};
 
-#[cfg(not(HAVE_EVENTFD))]
-use crate::fds::{make_autoclose_pipes, make_fd_nonblocking};
-#[cfg(HAVE_EVENTFD)]
-use libc::{EFD_CLOEXEC, EFD_NONBLOCK};
+cfg_if!(
+    if #[cfg(have_eventfd)] {
+        use libc::{EFD_CLOEXEC, EFD_NONBLOCK};
+    } else {
+        use crate::fds::{make_autoclose_pipes, make_fd_nonblocking};
+    }
+);
 
 /// An event signaller implemented using a file descriptor, so it can plug into
 /// [`select()`](libc::select).
@@ -33,7 +36,7 @@ use libc::{EFD_CLOEXEC, EFD_NONBLOCK};
 pub struct FdEventSignaller {
     // Always the read end of the fd; maybe the write end as well.
     fd: OwnedFd,
-    #[cfg(not(HAVE_EVENTFD))]
+    #[cfg(not(have_eventfd))]
     write: OwnedFd,
 }
 
@@ -41,29 +44,28 @@ impl FdEventSignaller {
     /// The default constructor will abort on failure (fd exhaustion).
     /// This should only be used during startup.
     pub fn new() -> Self {
-        #[cfg(HAVE_EVENTFD)]
-        {
-            // Note we do not want to use EFD_SEMAPHORE because we are binary (not counting) semaphore.
-            let fd = unsafe { libc::eventfd(0, EFD_CLOEXEC | EFD_NONBLOCK) };
-            if fd < 0 {
-                perror("eventfd");
-                exit_without_destructors(1);
-            }
-            Self {
-                fd: unsafe { OwnedFd::from_raw_fd(fd) },
-            }
-        }
-        #[cfg(not(HAVE_EVENTFD))]
-        {
-            // Implementation using pipes.
-            let Ok(pipes) = make_autoclose_pipes() else {
-                exit_without_destructors(1);
-            };
-            make_fd_nonblocking(pipes.read.as_raw_fd()).unwrap();
-            make_fd_nonblocking(pipes.write.as_raw_fd()).unwrap();
-            Self {
-                fd: pipes.read,
-                write: pipes.write,
+        cfg_if! {
+            if #[cfg(have_eventfd)] {
+                // Note we do not want to use EFD_SEMAPHORE because we are binary (not counting) semaphore.
+                let fd = unsafe { libc::eventfd(0, EFD_CLOEXEC | EFD_NONBLOCK) };
+                if fd < 0 {
+                    perror("eventfd");
+                    exit_without_destructors(1);
+                }
+                Self {
+                    fd: unsafe { OwnedFd::from_raw_fd(fd) },
+                }
+            } else {
+                // Implementation using pipes.
+                let Ok(pipes) = make_autoclose_pipes() else {
+                    exit_without_destructors(1);
+                };
+                make_fd_nonblocking(pipes.read.as_raw_fd()).unwrap();
+                make_fd_nonblocking(pipes.write.as_raw_fd()).unwrap();
+                Self {
+                    fd: pipes.read,
+                    write: pipes.write,
+                }
             }
         }
     }
@@ -80,16 +82,19 @@ impl FdEventSignaller {
         // If we are using eventfd, we want to read a single uint64.
         // If we are using pipes, read a lot; note this may leave data on the pipe if post has been
         // called many more times. In no case do we care about the data which is read.
-        #[cfg(HAVE_EVENTFD)]
-        let mut buff = [0_u64; 1];
-        #[cfg(not(HAVE_EVENTFD))]
-        let mut buff = [0_u8; 1024];
+        cfg_if!(
+            if #[cfg(have_eventfd)] {
+                let mut buff = [0_u64; 1];
+            } else {
+                let mut buff = [0_u8; 1024];
+            }
+        );
         let mut ret;
         loop {
             ret = unsafe {
                 libc::read(
                     self.read_fd(),
-                    &mut buff as *mut _ as *mut c_void,
+                    buff.as_mut_ptr().cast(),
                     std::mem::size_of_val(&buff),
                 )
             };
@@ -107,10 +112,13 @@ impl FdEventSignaller {
     /// This retries on EINTR.
     pub fn post(&self) {
         // eventfd writes uint64; pipes write 1 byte.
-        #[cfg(HAVE_EVENTFD)]
-        let c = 1_u64;
-        #[cfg(not(HAVE_EVENTFD))]
-        let c = 1_u8;
+        cfg_if!(
+            if #[cfg(have_eventfd)] {
+                let c = 1_u64;
+            } else {
+                let c = 1_u8;
+            }
+        );
         let mut ret;
         loop {
             let bytes = c.to_ne_bytes();
@@ -146,10 +154,13 @@ impl FdEventSignaller {
 
     /// Return the fd to write to.
     fn write_fd(&self) -> RawFd {
-        #[cfg(HAVE_EVENTFD)]
-        return self.fd.as_raw_fd();
-        #[cfg(not(HAVE_EVENTFD))]
-        return self.write.as_raw_fd();
+        cfg_if! {
+            if #[cfg(have_eventfd)] {
+                self.fd.as_raw_fd()
+            } else {
+                self.write.as_raw_fd()
+            }
+        }
     }
 }
 
@@ -173,13 +184,13 @@ impl From<u64> for FdMonitorItemId {
 /// The callback type used by [`FdMonitorItem`]. It is passed a mutable reference to the
 /// `FdMonitorItem`'s [`FdMonitorItem::fd`]. If the fd is closed, the callback will not
 /// be invoked again.
-pub type Callback = Box<dyn Fn(&mut AutoCloseFd) + Send + Sync>;
+pub type Callback = Box<dyn Fn(&mut Option<OwnedFd>) + Send + Sync>;
 
 /// An item containing an fd and callback, which can be monitored to watch when it becomes readable
 /// and invoke the callback.
 pub struct FdMonitorItem {
     /// The fd to monitor
-    fd: AutoCloseFd,
+    fd: Option<OwnedFd>,
     /// A callback to be invoked when the fd is readable, or for another reason given by the wake reason.
     /// If the fd is invalid on return from the function, then the item is removed from the [`FdMonitor`] set.
     callback: Callback,
@@ -237,12 +248,13 @@ struct BackgroundFdMonitor {
 
 impl FdMonitor {
     /// Add an item to the monitor. Returns the [`FdMonitorItemId`] assigned to the item.
-    pub fn add(&self, fd: AutoCloseFd, callback: Callback) -> FdMonitorItemId {
-        assert!(fd.is_valid());
-
+    pub fn add(&self, fd: OwnedFd, callback: Callback) -> FdMonitorItemId {
         let item_id = self.last_id.fetch_add(1, Ordering::Relaxed) + 1;
         let item_id = FdMonitorItemId(item_id);
-        let item: FdMonitorItem = FdMonitorItem { fd, callback };
+        let item: FdMonitorItem = FdMonitorItem {
+            fd: Some(fd),
+            callback,
+        };
         let start_thread = {
             // Lock around a local region
             let mut data = self.data.lock().expect("Mutex poisoned!");
@@ -258,7 +270,7 @@ impl FdMonitor {
         };
 
         if start_thread {
-            FLOG!(fd_monitor, "Thread starting");
+            flog!(fd_monitor, "Thread starting");
             let background_monitor = BackgroundFdMonitor {
                 data: Arc::clone(&self.data),
                 change_signaller: Arc::clone(&self.change_signaller),
@@ -274,11 +286,18 @@ impl FdMonitor {
         item_id
     }
 
+    pub fn with_fd(&self, item_id: FdMonitorItemId, cb: impl FnOnce(BorrowedFd)) {
+        let data = self.data.lock().expect("Mutex poisoned!");
+        if let Some(fd) = &data.items.get(&item_id).unwrap().fd {
+            cb(fd.as_fd());
+        }
+    }
+
     /// Remove an item from the monitor and return its file descriptor.
     /// Note we may remove an item whose fd is currently being waited on in select(); this is
     /// considered benign because the underlying item will no longer be present and so its
     /// callback will not be invoked.
-    pub fn remove_item(&self, item_id: FdMonitorItemId) -> AutoCloseFd {
+    pub fn remove_item(&self, item_id: FdMonitorItemId) -> Option<OwnedFd> {
         assert!(item_id.0 > 0, "Invalid item id!");
         let mut data = self.data.lock().expect("Mutex poisoned!");
         let removed = data.items.remove(&item_id).expect("Item ID not found");
@@ -357,9 +376,8 @@ impl BackgroundFdMonitor {
             item_ids.clear();
             item_ids.reserve(data.items.len());
             for (item_id, item) in &data.items {
-                let fd = item.fd.as_raw_fd();
-                if fd >= 0 {
-                    fds.add(fd);
+                if let Some(fd) = &item.fd {
+                    fds.add(fd.as_raw_fd());
                     item_ids.push(*item_id);
                 }
             }
@@ -387,11 +405,13 @@ impl BackgroundFdMonitor {
             //
             // Note that WSLv1 doesn't throw EBADF if the fd is closed is mid-select.
             drop(data);
-            let ret =
-                fds.check_readable(timeout.map(Timeout::Duration).unwrap_or(Timeout::Forever));
+            let ret = fds.check_readable(timeout.map_or(Timeout::Forever, Timeout::Duration));
             // Cygwin reports ret < 0 && errno == 0 as success.
             let err = errno().0;
-            if ret < 0 && !matches!(err, libc::EINTR | libc::EBADF) && !(cfg!(cygwin) && err == 0) {
+            if ret < 0
+                && !matches!(err, libc::EINTR | libc::EBADF | libc::EAGAIN)
+                && !(cfg!(cygwin) && err == 0)
+            {
                 // Surprising error
                 perror("select");
             }
@@ -407,7 +427,7 @@ impl BackgroundFdMonitor {
                     // Note there is no risk of an ABA problem because ItemIDs are never recycled.
                     continue;
                 };
-                if fds.test(item.fd.as_raw_fd()) {
+                if item.fd.as_ref().is_some_and(|fd| fds.test(fd.as_raw_fd())) {
                     item.service();
                 }
             }
@@ -427,7 +447,7 @@ impl BackgroundFdMonitor {
                         data.running,
                         "Thread should be running because we're that thread"
                     );
-                    FLOG!(fd_monitor, "Thread exiting");
+                    flog!(fd_monitor, "Thread exiting");
                     data.running = false;
                     break;
                 }
@@ -456,7 +476,7 @@ mod tests {
     use portable_atomic::AtomicU64;
     use std::fs::File;
     use std::io::Write;
-    use std::os::fd::{AsRawFd, IntoRawFd, OwnedFd};
+    use std::os::fd::{AsRawFd, OwnedFd};
     #[cfg(target_has_atomic = "64")]
     use std::sync::atomic::AtomicU64;
     use std::sync::atomic::{AtomicUsize, Ordering};
@@ -468,7 +488,7 @@ mod tests {
 
     use crate::fd_monitor::{FdEventSignaller, FdMonitor};
     use crate::fd_readable_set::{FdReadableSet, Timeout};
-    use crate::fds::{AutoCloseFd, AutoClosePipes, make_autoclose_pipes};
+    use crate::fds::{AutoClosePipes, make_autoclose_pipes};
     use crate::tests::prelude::*;
 
     /// Helper to make an item which counts how many times its callback was invoked.
@@ -504,25 +524,25 @@ mod tests {
             let result = Arc::new(result);
             let callback = {
                 let result = Arc::clone(&result);
-                move |fd: &mut AutoCloseFd| result.callback(fd)
+                move |fd: &mut Option<OwnedFd>| result.callback(fd)
             };
-            let fd = AutoCloseFd::new(pipes.read.into_raw_fd());
+            let fd = pipes.read;
             let item_id = monitor.add(fd, Box::new(callback));
             result.item_id.store(u64::from(item_id), Ordering::Relaxed);
 
             result
         }
 
-        fn callback(&self, fd: &mut AutoCloseFd) {
+        fn callback(&self, fd: &mut Option<OwnedFd>) {
             let mut buf = [0u8; 1024];
-            let res = nix::unistd::read(&fd, &mut buf);
+            let res = nix::unistd::read(fd.as_ref().unwrap(), &mut buf);
             let amt = res.expect("read error!");
             self.length_read.fetch_add(amt, Ordering::Relaxed);
             let was_closed = amt == 0;
 
             self.total_calls.fetch_add(1, Ordering::Relaxed);
             if was_closed || self.always_close {
-                fd.close();
+                drop(fd.take());
             }
         }
 
