@@ -1,4 +1,4 @@
-use crate::flog::FLOG;
+use crate::flog::flog;
 use std::cell::{Ref, RefMut};
 use std::sync::MutexGuard;
 use std::sync::atomic::{AtomicBool, AtomicPtr, Ordering};
@@ -35,9 +35,7 @@ pub struct AtomicRef<T: ?Sized + 'static>(AtomicPtr<&'static T>);
 
 impl<T: ?Sized> AtomicRef<T> {
     pub const fn new(value: &'static &'static T) -> Self {
-        Self(AtomicPtr::new(
-            value as *const &'static T as *mut &'static T,
-        ))
+        Self(AtomicPtr::new(std::ptr::from_ref(value).cast_mut()))
     }
 
     pub fn load(&self) -> &'static T {
@@ -45,10 +43,8 @@ impl<T: ?Sized> AtomicRef<T> {
     }
 
     pub fn store(&self, value: &'static &'static T) {
-        self.0.store(
-            value as *const &'static T as *mut &'static T,
-            Ordering::Relaxed,
-        )
+        self.0
+            .store(std::ptr::from_ref(value).cast_mut(), Ordering::Relaxed)
     }
 }
 
@@ -56,7 +52,7 @@ pub struct DebugRef<'a, T>(Ref<'a, T>);
 
 impl<'a, T> DebugRef<'a, T> {
     pub fn new(r: Ref<'a, T>) -> Self {
-        FLOG!(
+        flog!(
             refcell,
             "CREATE DebugRef",
             std::backtrace::Backtrace::capture()
@@ -67,7 +63,7 @@ impl<'a, T> DebugRef<'a, T> {
 
 impl<'a, T> Drop for DebugRef<'a, T> {
     fn drop(&mut self) {
-        FLOG!(
+        flog!(
             refcell,
             "DROP DebugRef",
             std::backtrace::Backtrace::capture()
@@ -86,7 +82,7 @@ pub struct DebugRefMut<'a, T>(RefMut<'a, T>);
 
 impl<'a, T> DebugRefMut<'a, T> {
     pub fn new(r: RefMut<'a, T>) -> Self {
-        FLOG!(
+        flog!(
             refcell,
             "CREATE DebugRefMut",
             std::backtrace::Backtrace::capture()
@@ -97,7 +93,7 @@ impl<'a, T> DebugRefMut<'a, T> {
 
 impl<'a, T> Drop for DebugRefMut<'a, T> {
     fn drop(&mut self) {
-        FLOG!(
+        flog!(
             refcell,
             "DROP DebugRefMut",
             std::backtrace::Backtrace::capture()
@@ -121,7 +117,7 @@ pub struct DebugMutexGuard<'a, T>(MutexGuard<'a, T>);
 
 impl<'a, T> DebugMutexGuard<'a, T> {
     pub fn new(r: MutexGuard<'a, T>) -> Self {
-        FLOG!(
+        flog!(
             refcell,
             "CREATE DebugMutexGuard",
             std::backtrace::Backtrace::capture()
@@ -132,7 +128,7 @@ impl<'a, T> DebugMutexGuard<'a, T> {
 
 impl<'a, T> Drop for DebugMutexGuard<'a, T> {
     fn drop(&mut self) {
-        FLOG!(
+        flog!(
             refcell,
             "DROP DebugMutexGuard",
             std::backtrace::Backtrace::capture()
